/**
 * 
 */
package org.arabicfedora.io;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;

import org.arabicfedora.POFile;
import org.arabicfedora.POMessage;

/**
 * This class is responsible for loading a PO file into a POFile instance. It
 * can take any charset name documented in java.nio.charset.Charset.
 * 
 * @author Mohammad Ghoniem
 */
public class POFileLoader {

	//	default encoding is set to UTF-8,
	// 	check class Charset for other possiblities
	private String charsetName = "UTF-8"; 

	private POFile pofile = null;
	
	private static final int MULTILINE_NONE		= -1;
	private static final int MULTILINE_MSGID 	= 0;
	private static final int MULTILINE_MSGSTR 	= 1;
	private int mode = MULTILINE_NONE;

	public POFileLoader() {
		pofile = new POFile();
	}

	public void load(File file) {
		this.pofile.clear();
		System.out.print("Start parsing...");
		try {
			FileInputStream fis = new FileInputStream(file);
			InputStreamReader isr = new InputStreamReader(fis, charsetName);
			BufferedReader reader = new BufferedReader(isr);

			String line = reader.readLine();
			String id = null;
			String str = null;
			String comments = "";

			int start, end = -1;
			while (line != null) {
				System.out.print(".");
				// retrieves comments
				if (isComment(line)) {
						comments +=(comments.length()>0)?("\n"+line):(line);
				}
				// retrieves original strings
				else if (line.startsWith("msgid")) {
					start = line.indexOf("\"");
					end = line.lastIndexOf("\"");
					id = line.substring(start + 1, end);
					mode = MULTILINE_MSGID;
				}
				// retrieves translated strings
				else if (line.startsWith("msgstr")) {
					start = line.indexOf("\"");
					end = line.lastIndexOf("\"");
					str = line.substring(start + 1, end);
					mode = MULTILINE_MSGSTR;
				// handles multi-line messages
				} else if (line.startsWith("\"")){
					start = 0;
					end = line.lastIndexOf("\"");
					switch(mode){
					case MULTILINE_MSGID : id+=(line.substring(start + 1, end)+"\n");
					case MULTILINE_MSGSTR : str+=(line.substring(start + 1, end)+"\n");
					}
				} else {
					mode = MULTILINE_NONE;
				}
				// sets up a POMessage instance and stores it
				if (id != null && str != null && mode==MULTILINE_NONE) {
					pofile.addMessage(new POMessage(id.intern(), str
							.intern(), comments.intern()));
					id = null;
					str = null;
					comments = "";
				}
				start = -1;
				end = -1;
				line = reader.readLine();
			}
			System.out.println("\nDone.");
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}

	public void print() {
		pofile.print();
	}

	/**
	 * @see java.nio.charset.Charset
	 * @return
	 */
	public String getCharsetName() {
		return charsetName;
	}

	/**
	 * @see java.nio.charset.Charset
	 * @param charsetName
	 */
	public void setCharsetName(String charsetName) {
		this.charsetName = charsetName;
	}

	public POFile getPofile() {
		return pofile;
	}

	public void setPofile(POFile pofile) {
		this.pofile = pofile;
	}

	public static boolean isTranslatorComment(String str) {
		return str.startsWith("# ");
	}

	public static boolean isAutoGeneratedComment(String str) {
		return str.startsWith("#.");
	}

	public static boolean isFlag(String str) {
		return str.startsWith("#,");
	}

	public static boolean isReference(String str) {
		return str.startsWith("#:");
	}

	public static boolean isComment(String str) {
		return str.startsWith("#");
	}

}
